function [ targetFactor, pathFactor, measuresOfFit,F ] = factor_rotation( assetPriceChanges )
%FACTOR_ROTATION: Calculates principal components and rotates factors to
%get target and path factors
% See GSS (2005) IJCB for details
% Using code from Campbell et al (2012) Brookings Paper

%[inverseFactorLoadings,estimatedFactors,factorVariances]=princomp(zscore(assetPriceChanges));
[inverseFactorLoadings,estimatedFactors,factorVariances]=pca(zscore(assetPriceChanges));

%The inverse factor loadings and estimated factors require two
%modifications before proceeding. First, we need to standardize the factors
%to have unit variance. Therefore, the factor loadings should be premultiplied
%by the factors' standard deviations. We perform this multiplication as
%well as the inversion of |inverseFactorLoadings| in one step.
factorLoadings=diag(sqrt(factorVariances))/inverseFactorLoadings;
estimatedFactors=estimatedFactors/diag(sqrt(factorVariances));

%Next, we need to calculate the factor loadings for the original asset
%price changes rather than for their standardized values. For this, we
%postmultiply the factor loadings by the diagonal matrix of asset price
%changes' standard deviations.
factorLoadings=factorLoadings*diag(std(assetPriceChanges));

%Following GSS, we focus on the first two principle components. To adhere
%to their notation, we put these two components in |F| and the
%corresponding factor loadings in |gamma|.
gamma=factorLoadings(1:2,:);
F=estimatedFactors(:,1:2);

alpha_1 = sign(gamma(1,1))/sqrt(1+(gamma(2,1)/gamma(1,1))^2);
alpha_2 = alpha_1*gamma(2,1)/gamma(1,1);

beta_2 = sign(det(gamma(1:2,1:2)))*alpha_1;
beta_1 = -beta_2*gamma(2,1)/gamma(1,1);
U = [alpha_1 beta_1; alpha_2 beta_2];
Z=F*U;


%Hence $S$ scales the factors and loadings such that the effect of the second factor on the fourth (not fifth as in GSS) interest rate is unity, as desired.
Shat=eye(2);
Shat(1,1)=gamma(1,1)/alpha_1;
Shat(2,2)=det(gamma(1:2,1:2))/(gamma(1,1)*beta_2*(1+(gamma(2,1)/gamma(1,1))^2));
Zhat = Z*Shat;
gammaStarhat=Shat\(U\gamma);

S=eye(2); 
S(2,2)=gammaStarhat(2,4); 
SInverse=eye(2); 
SInverse(2,2)=1/gammaStarhat(2,4); 

gammaStar=SInverse*gammaStarhat; 
Zstar    =Zhat*S; 

targetFactor=Zstar(:,1);
pathFactor=Zstar(:,2);

fittedValues=Zstar*gammaStar;

targetFactorPrices = targetFactor*gammaStar(1,:);
pathFactorPrices = pathFactor*gammaStar(2,:);

measuresOfFit=zeros(size(fittedValues,2),3);
measuresOfFit(:,1)=(var(targetFactorPrices)./var(assetPriceChanges))';
measuresOfFit(:,2)=(var(pathFactorPrices)./var(assetPriceChanges))';
measuresOfFit(:,3)=1-(var(fittedValues)./var(assetPriceChanges))';


end

